local addonName, HeyBarbaruivaNameplates = ...

--[[ Defaults ]]-- 

HeyBarbaruivaNameplates.Defaults = {
	OnlyShowOwnBuffs = 1,
	ShowBuffs = 1,
	Version = GetAddOnMetadata(addonName, "Version")
}


--[[ Interface options panel ]]-- 

--[[
HeyBarbaruivaNameplates.OptionsPanel = HeyBarbaruivaNameplates:CreateOptionsPanel();
local optionsPanel = HeyBarbaruivaNameplates.OptionsPanel;
optionsPanel.savedVariablesName = "EngravedOptions";
-- optionsPanel.defaults set in SetClassDefaults()
optionsPanel.defaultsFunc = function()
	local runeFrame, options = EngravedRuneFrame, EngravedOptions;
	runeFrame:SetPosition(options.RuneFramePosition);
	runeFrame:SetSize(options.RuneFrameSize);
	runeFrame:SetRunePositions(options.RunePositions);
	runeFrame:SetRuneSizes(options.RuneSizes);
	runeFrame:SetLocked(options.IsLocked);
end;
]]--



-- HeyBarbaruivaNameplates_Options.lua

-- Localization file should be loaded before this
-- "Functionality" and MyAddon_Options.xml files should be later

local addOnName = "HeyBarbaruivaNameplates";
-- Should be name of AddOn folder and TOC file

local DefaultSettings = {
	-- Use 1/0 instead of true/false
	onlyShowOwnBuffs = 1;
	showBuffs = 1;
	version = GetAddOnMetadata(addOnName, "Version"),
}

-- AddOn Settings are stored as MyAddOn_Settings (HeyBarbaruivaNameplates_Settings, for example). 
-- Make sure to list in TOC file under "## SavedVariablesPerCharacter:" or "## SavedVariables:"
-- Retrieve settings using MyAddOn:GetSetting(setting)

-- MYADDON namespace defined in MyAddOn_Localization.lua file
local strings = HeyBarbaruivaNameplates; 
local UIPanelOptions = {
	showBuffs = {
		text    = strings.SHOW_BUFFS, 
		tooltip = strings.TOOLTIP_SHOW_BUFFS 
	}, 
	onlyShowOwnBuffs = {
		text    = strings.ONLY_SHOW_OWN_BUFFS, 
		tooltip = strings.TOOLTIP_ONLY_SHOW_OWN_BUFFS 
	}
}


--[[ AddOn-independent code below here ]]--

-- Define AddOn namespace
_G[addOnName] = {};
local MyAddOn = _G[addOnName];

-- String namespace should already be defined in localization file
local MYADDON = _G[string.upper(addOnName)];

-- AddOn title (can have spaces)
local addOnTitle = GetAddOnMetadata(addOnName, "Title")


-- [[ Saved variables ]] --

local function CopyTable (source)
	local copy = {};
	for key, value in pairs(source) do
		if ( type(value) == "table" ) then
			copy[key] = CopyTable(value);
		else
			copy[key] = value;
		end
	end
	return copy;
end

local function UpdateTable (source, destination)
	for key, value in pairs(source) do
		if ( type(value) == "table" ) then
			if ( not destination[key] ) then
				destination[key] = {};
			end
			destination[key] = UpdateTable(value, destination[key]);
		elseif ( not destination[key] ) then
			destination[key] = value;
		end
	end
	return destination;
end

-- AddOn settings called CVars to parallel Blizzard code 
-- even though they're not real console variables

local function UpdateSettings ()
	local settings = _G[addOnName.."_Settings"]
	settings = UpdateTable(DefaultSettings, settings);
	settings["version"] = GetAddOnMetadata(addOnName, "Version");
end

function MyAddOn:GetSetting (cvar)
	-- Called in "functionality" files
	local settings = _G[addOnName.."_Settings"]
	return settings[cvar];
end

local function GetDefaultSetting (cvar)
	return DefaultSettings[cvar];
end

local function SetSetting (cvar, value)
	local settings = _G[addOnName.."_Settings"]
	settings[cvar] = value;
end


-- Code below derived from default UI (OptionsPanelTemplates, InterfaceOptionsPanels)

-- [[ StaticPopup alert for reloading the user interface ]] --

StaticPopupDialogs[string.upper(addOnName).."_RELOAD_ALERT"] = {
	text = MYADDON.RELOAD_ALERT,
	button1 = RELOADUI,
	button2 = IGNORE_DIALOG,
	OnAccept = function(self, data) ReloadUI(); end,
	timeout = 0,
	whileDead = 1,
	hideOnEscape = 1
};


-- [[ Options panel ]] --

local function CheckButton_SetNewValue (checkButton)
	local setting = 0;
	if ( checkButton:GetChecked() ) then
		setting = 1;
	end
	if ( setting == checkButton.value ) then
		checkButton.newValue = nil;
	else
		checkButton.newValue = setting;
	end
	if ( checkButton.shouldReload ) then
		if ( checkButton.newValue ) then
			checkButton.Reload:Show();
		else
			checkButton.Reload:Hide();
		end
	end
end

local function CheckButton_Refresh (checkButton)
	local value;
	if ( checkButton.cvar ) then
		value = MyAddOn:GetSetting(checkButton.cvar);
	elseif ( checkButton.GetValue ) then
		value = checkButton:GetValue();
	end
	if ( value ) then
		if ( value == 1 ) then
			checkButton:SetChecked(true);
		else
			checkButton:SetChecked(false);
		end
		checkButton.value = value;
	end
end

local function OptionsPanelControl_Okay (control)
	if ( control.newValue ) then
		if ( control.value ~= control.newValue ) then
			control:SetValue(control.newValue);
			control.value = control.newValue;
			control.newValue = nil;
		end
	elseif ( control.value ) then
		if ( control:GetValue() ~= control.value ) then
			control:SetValue(control.value);
		end
	end	
end

local function OptionsPanelControl_Cancel (control)
	if ( control.newValue ) then
		if ( control.value and control.value ~= control.newValue ) then
			-- we need to force-set the value here just in case the control was doing dynamic updating
			control:SetValue(control.value);
			control.newValue = nil;
		end
	elseif ( control.value ) then
		if ( control:GetValue() ~= control.value ) then
			control:SetValue(control.value);
		end
	end
	if ( control.shouldReload ) then
		control.Reload:Hide();
	end
end

local function OptionsPanelControl_Default (control)
	if ( control.defaultValue and control.value ~= control.defaultValue ) then
		control:SetValue(control.defaultValue);
		control.value = control.defaultValue;
		control.newValue = nil;
		if ( control.shouldReload ) then
			control.Reload:Show();
		end
	end
end

local function OptionsPanelControl_Refresh (control)
	if ( control.RefreshValue ) then
		control:RefreshValue();
	end
	if ( control.type == CONTROLTYPE_CHECKBOX ) then
		CheckButton_Refresh(control);
	end
end

local function OptionsPanelControl_Setup (control)
	if ( control.type == CONTROLTYPE_CHECKBOX ) then			
		if ( control.cvar ) then
			local value = MyAddOn:GetSetting(control.cvar);
			control.value = value;
			control.GetValue = function(self) return MyAddOn:GetSetting(self.cvar); end
			control.SetValue = function(self, value) self.value = value; SetSetting(self.cvar, value); end
		elseif ( control.GetValue ) then
			local value = control:GetValue();
			if ( value ) then
				control.value = value;
			else
				control.value = "0";
			end
			control.SetValue = function(self, value) self.value = value; end;
		end
	end
end

local function OptionsPanel_Okay (self)
	self.shouldReload = nil;
	for _, control in next, self.controls do
		OptionsPanelControl_Okay(control);
		if ( control.shouldReload and control.Reload:IsShown() ) then
			self.shouldReload = true;
			control.Reload:Hide();
		end
	end
	if ( self.shouldReload ) then
		StaticPopup_Show(string.upper(addOnName).."_RELOAD_ALERT");
		self.shouldReload = nil;
	end
	MyAddOn:Update();  -- This function should be defined in the "functionality" lua file
end

local function OptionsPanel_Cancel (self)
	for _, control in next, self.controls do
		OptionsPanelControl_Cancel(control);
	end
end

local function OptionsPanel_Default (self)
	for _, control in next, self.controls do
		OptionsPanelControl_Default(control);
	end
end

local function OptionsPanel_Refresh (self)
	for _, control in next, self.controls do
		OptionsPanelControl_Refresh(control);
	end
end

local function OptionsPanel_OnEvent (frame, event, ...)
	if ( event == "ADDON_LOADED" ) then
		local settings = _G[addOnName.."_Settings"];
		if ( not settings ) then 
			_G[addOnName.."_Settings"] = CopyTable(DefaultSettings);
		-- elseif ( settings["version"] ~= GetAddOnMetadata(addOnName, "Version") ) then
		else 
			UpdateSettings();
		end
	elseif ( event == "PLAYER_ENTERING_WORLD" ) then
		if ( frame.options and frame.controls ) then
			local entry;
			for i, control in next, frame.controls do
				entry = frame.options[(control.cvar or control.label)];
				if ( entry ) then
					if ( entry.text ) then
						control.tooltipText = entry.tooltip;
						local text = _G[control:GetName() .. "Text"];
						if ( text ) then
							text:SetText(entry.text);
						end
					end
					if ( control.cvar ) then
						if ( control.type == CONTROLTYPE_CHECKBOX ) then
							control.defaultValue = control.defaultValue or GetDefaultSetting(control.cvar);
						end
					end
					OptionsPanelControl_Setup(control);
				end
			end
		end
	end
end


-- [[ Functions called by MyAddOn_Options.xml ]] --

function MyAddOn.OptionsPanel_OnLoad(frame)
	frame.name = addOnTitle;
	frame.options = UIPanelOptions;
	frame.okay = OptionsPanel_Okay;
	frame.cancel = OptionsPanel_Cancel;
	frame.default = OptionsPanel_Default;
	frame.refresh = OptionsPanel_Refresh;
	InterfaceOptions_AddCategory(frame);

	frame:RegisterEvent("ADDON_LOADED");
	frame:RegisterEvent("PLAYER_ENTERING_WORLD");
	frame:SetScript("OnEvent", OptionsPanel_OnEvent);

	_G[frame:GetName().."Title"]:SetText(addOnTitle.." v"..GetAddOnMetadata(addOnName, "Version"));
	_G[frame:GetName().."SubText"]:SetText(MYADDON.SUBTEXT);
end

function MyAddOn.OptionsPanel_RegisterControl(control, parentFrame)
	if ( ( not parentFrame ) or ( not control ) ) then
		return;
	end
	parentFrame.controls = parentFrame.controls or {};
	tinsert(parentFrame.controls, control);
	-- Use the panel's OnEvent handler to wait and setup the control after game data is loaded
end

function MyAddOn.OptionsPanel_CheckButton_OnClick (checkButton)
	CheckButton_SetNewValue(checkButton);
end





